package sun.misc;

import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.Map;
import java.util.Vector;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.io.File;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.HashMap;

public class JarIndex
{
    private HashMap<String, LinkedList<String>> indexMap;
    private HashMap<String, LinkedList<String>> jarMap;
    private String[] jarFiles;
    public static final String INDEX_NAME = "META-INF/INDEX.LIST";
    private static final boolean metaInfFilenames;
    
    public JarIndex() {
        this.indexMap = new HashMap();
        this.jarMap = new HashMap();
    }
    
    public JarIndex(InputStream inputStream) throws IOException {
        this();
        this.read(inputStream);
    }
    
    public JarIndex(String[] jarFiles) throws IOException {
        this();
        this.parseJars(this.jarFiles = jarFiles);
    }
    
    public static JarIndex getJarIndex(JarFile jarFile) throws IOException {
        return getJarIndex(jarFile, null);
    }
    
    public static JarIndex getJarIndex(JarFile jarFile, MetaIndex metaIndex) throws IOException {
        JarIndex jarIndex = null;
        if (metaIndex != null && !metaIndex.mayContain("META-INF/INDEX.LIST")) {
            return null;
        }
        JarEntry jarEntry = jarFile.getJarEntry("META-INF/INDEX.LIST");
        if (jarEntry != null) {
            jarIndex = new JarIndex(jarFile.getInputStream(jarEntry));
            return jarIndex;
        }
        return jarIndex;
    }
    
    public String[] getJarFiles() {
        return this.jarFiles;
    }
    
    private void addToList(String s, String e, HashMap<String, LinkedList<String>> hashMap) {
        LinkedList<String> list = (LinkedList)hashMap.get(s);
        if (list == null) {
            LinkedList<String> value = new LinkedList();
            value.add(e);
            hashMap.put(s, value);
        }
        else if (!list.contains(e)) {
            list.add(e);
        }
    }
    
    public LinkedList<String> get(String key) {
        LinkedList<Object> list;
        int lastIndex;
        if ((list = (LinkedList)this.indexMap.get(key)) == null && (lastIndex = key.lastIndexOf("/")) != -1) {
            list = (LinkedList)this.indexMap.get(key.substring(0, lastIndex));
            return list;
        }
        return list;
    }
    
    public void add(String s, String s2) {
        int lastIndex;
        String substring;
        if ((lastIndex = s.lastIndexOf("/")) != -1) {
            substring = s.substring(0, lastIndex);
        }
        else {
            substring = s;
        }
        this.addMapping(substring, s2);
    }
    
    private void addMapping(String s, String s2) {
        this.addToList(s, s2, this.indexMap);
        this.addToList(s2, s, this.jarMap);
    }
    
    private void parseJars(String[] array) throws IOException {
        if (array == null) {
            return;
        }
        for (int i = 0; i < array.length; ++i) {
            String s = array[i];
            ZipFile o = new ZipFile(s.replace('/', File.separatorChar));
            Enumeration<Object> entries = o.entries();
            while (entries.hasMoreElements()) {
                ZipEntry zipEntry = (ZipEntry)entries.nextElement();
                String name = zipEntry.getName();
                if (name.equals("META-INF/") || name.equals("META-INF/INDEX.LIST")) {
                    continue;
                }
                if (name.equals("META-INF/MANIFEST.MF")) {
                    continue;
                }
                if (!JarIndex.metaInfFilenames || !name.startsWith("META-INF/")) {
                    this.add(name, s);
                }
                else if (!zipEntry.isDirectory()) {
                    this.addMapping(name, s);
                }
                else {
                    continue;
                }
            }
            o.close();
        }
        return;
    }
    
    public void write(OutputStream out) throws IOException {
        BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(out, "UTF8"));
        bufferedWriter.write("JarIndex-Version: 1.0\n\n");
        if (this.jarFiles != null) {
            for (int i = 0; i < this.jarFiles.length; ++i) {
                String s = this.jarFiles[i];
                bufferedWriter.write(s + "\n");
                LinkedList<Object> list = (LinkedList)this.jarMap.get(s);
                if (list != null) {
                    Iterator<Object> iterator = list.iterator();
                    while (iterator.hasNext()) {
                        bufferedWriter.write((String)iterator.next() + "\n");
                    }
                }
                bufferedWriter.write("\n");
            }
            bufferedWriter.flush();
        }
    }
    
    public void read(InputStream in) throws IOException {
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(in, "UTF8"));
        String e = null;
        Vector<String> vector = new Vector();
        String s;
        while ((s = bufferedReader.readLine()) != null && !s.endsWith(".jar")) {}
        while (s != null) {
            if (s.length() != 0) {
                if (s.endsWith(".jar")) {
                    e = s;
                    vector.add(e);
                }
                else {
                    this.addMapping(s, e);
                }
            }
            s = bufferedReader.readLine();
        }
        this.jarFiles = (String[])vector.toArray(new String[vector.size()]);
    }
    
    public void merge(JarIndex jarIndex, String s) {
        for (Map.Entry entry : this.indexMap.entrySet()) {
            String s2 = (String)entry.getKey();
            for (String concat : (LinkedList)entry.getValue()) {
                if (s != null) {
                    concat = s.concat(concat);
                }
                jarIndex.addMapping(s2, concat);
            }
        }
    }
    
    static {
        metaInfFilenames = "true".equals(AccessController.doPrivileged(new GetPropertyAction("sun.misc.JarIndex.metaInfFilenames")));
    }
}
